using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace Shisen_Sho {
    public partial class FrmMain : Form {
        //private Grid g;
        public const int NUM_HIGH_SCORES = 10;
        private int num_tiles_h = 24;
        private int num_tiles_v = 12;
        public FrmMain() {
            InitializeComponent();
            //g = Grid.make(5, 10);
            mnuGravity.Checked = Properties.Settings.Default.gravity;
            mnuOnlySolvable.Checked = Properties.Settings.Default.onlySolvable;
            if (Properties.Settings.Default.difficulty == 0.2) {
                adjustDifficulty(mnuDifficulty20);
            } else if (Properties.Settings.Default.difficulty == 0.5) {
                adjustDifficulty(mnuDifficulty50);
            } else if (Properties.Settings.Default.difficulty == 1) {
                adjustDifficulty(mnuDifficulty100);
            } else if (Properties.Settings.Default.difficulty == 1.5) {
                adjustDifficulty(mnuDifficulty150);
            }

            if (Properties.Settings.Default.dim == "14x6") {
                adjustDimensions(mnuDim14x6);
            } else if (Properties.Settings.Default.dim == "18x8") {
                adjustDimensions(mnuDim18x8);
            } else if (Properties.Settings.Default.dim == "24x12") {
                adjustDimensions(mnuDim24x12);
            } else if (Properties.Settings.Default.dim == "26x14") {
                adjustDimensions(mnuDim26x14);
            } else if (Properties.Settings.Default.dim == "30x16") {
                adjustDimensions(mnuDim30x16);
            }

            if (File.Exists(Application.StartupPath + "\\HighScores.xml")) {
                dsScores.ReadXml(Application.StartupPath + "\\HighScores.xml");
            }
            tssTime.Text = "Time: 0";
            board.init(num_tiles_v, num_tiles_h);

            board.tmrGame.Tick += new EventHandler(tmrGame_Tick);
        }

        void tmrGame_Tick(object sender, EventArgs e) {
            stMain.Items[0].Text = "Time: " + board.incGameTime();
        }

        private void btnHint_Click(object sender, EventArgs e) {
            if (!board.drawHint())
                MessageBox.Show("No more moves possible.", "Game Over", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private void FrmMain_Resize(object sender, EventArgs e) {
            btnExit.Left = this.Width - 116;
        }

        private void btnNewGame_Click(object sender, EventArgs e) {
            if (confirmNewGame())
                newGame();
        }

        private bool confirmNewGame() {
            return !board.isGameStarted() ||
                    MessageBox.Show("This will cancel the current game and start a new one. Are you sure?", "Confirm New Game",
                    MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes;
        }

        private void newGame() {
            stMain.Items[0].Text = "Time: 0";
            this.Cursor = Cursors.WaitCursor;
            board.setDimensions(num_tiles_v, num_tiles_h);
            board.reset();
            this.Cursor = Cursors.Default;
        }

        private void mnuGravity_Click(object sender, EventArgs e) {
            if (confirmNewGame()) {
                mnuGravity.Checked = !mnuGravity.Checked;
                Properties.Settings.Default.gravity = mnuGravity.Checked;
                newGame();
            }
        }

        private void FrmMain_FormClosed(object sender, FormClosedEventArgs e) {
            Properties.Settings.Default.Save();
            dsScores.WriteXml(Application.StartupPath + "\\HighScores.xml");
        }

        private void mnuOnlySolvable_Click(object sender, EventArgs e) {
            if (confirmNewGame()) {
                mnuOnlySolvable.Checked = !mnuOnlySolvable.Checked;
                Properties.Settings.Default.onlySolvable = mnuOnlySolvable.Checked;
                newGame();
            }
        }

        private void adjustDifficulty(ToolStripMenuItem mnu) {
            Properties.Settings.Default.difficulty = double.Parse(mnu.Tag.ToString());
            foreach (ToolStripMenuItem m in mnuDifficulty.DropDownItems) {
                m.Checked = m == mnu;
            }
            tssDiff.Text = "Difficulty: " + mnu.Text;
        }

        private void mnuBestTimes_Click(object sender, EventArgs e) {
            new FrmHighScores(dsScores).ShowDialog();
        }

        private void board_OnGameWon(object sender, EventArgs e) {
            if (!board.isCheated()) {
                int time = board.getGameTime();
                DataRow[] betterScores = dsScores.Tables["Scores"].Select("GridSize = '" +
                    Properties.Settings.Default.dim + "' and Difficulty = " +
                    Properties.Settings.Default.difficulty + " and Time <= " + time, "Time ASC");

                if (betterScores.Length < NUM_HIGH_SCORES) {
                    string player = FrmNamePrompt.showNamePrompt(this, dsScores, time);
                    if (player != null)
                        new FrmHighScores(dsScores).ShowDialog();
                }
            }
        }

        private void btnExit_Click(object sender, EventArgs e) {
            this.Close();
        }

        private void mnuNewGame_Click(object sender, EventArgs e) {
            btnNewGame_Click(btnNewGame, EventArgs.Empty);
        }

        private void mnuGetHint_Click(object sender, EventArgs e) {
            btnHint_Click(btnHint, EventArgs.Empty);
        }

        private void mnuExit_Click(object sender, EventArgs e) {
            this.Close();
        }

        private void board_OnGameLost(object sender, EventArgs e) {
            MessageBox.Show("No more moves possible.", "Game Over", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private void adjustDimensions(ToolStripMenuItem mnu) {
            string gridSize = mnu.Tag.ToString();
            int xLoc = gridSize.IndexOf('x');
            num_tiles_h = int.Parse(gridSize.Substring(0, xLoc));
            num_tiles_v = int.Parse(gridSize.Substring(xLoc + 1));
            Properties.Settings.Default.dim = gridSize;
            foreach (ToolStripMenuItem m in mnuGridSize.DropDownItems) {
                m.Checked = m == mnu;                    
            }
            tssGridSize.Text = "Grid Size: " + mnu.Text;
        }

        private void board_Resize(object sender, EventArgs e) {
            this.Width = board.Right + 20;
            this.Height = board.Bottom + 40 + pnButtons.Height + stMain.Height;
        }

        private void mnuAbout_Click(object sender, EventArgs e) {
            MessageBox.Show("Shisen-Sho clone by Sourcecode  .\r\n\r\nTile pictures stolen from the KDE version.", "About", MessageBoxButtons.OK, MessageBoxIcon.Information);
           
        }

        private void shisenBoard_OnCheatedChanged(object sender, EventArgs e) {
            tssCheated.Text = "Cheated: " + (board.isCheated() ? "Yes" : "No");
        }

        private void mnuDiff_Click(object sender, EventArgs e) {
            if (confirmNewGame()) {
                adjustDifficulty((ToolStripMenuItem)sender);
                newGame();
            }
        }

        private void mnuGridSize_Click(object sender, EventArgs e) {
            if (confirmNewGame()) {
                adjustDimensions((ToolStripMenuItem)sender);
                newGame();
            }
        }

        private void mnuHelp_Click(object sender, EventArgs e)
        {

        }

        private void sourceCodeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            System.Diagnostics.Process.Start("http://WWw.SourceCodes.ir");
        }
    }
}